package eu.starsong.ghidra.api;

public class ApiConstants {
    public static final String PLUGIN_VERSION = "v2.1.0";
    public static final int API_VERSION = 2010;
    public static final int DEFAULT_PORT = 8192;
    public static final int MAX_PORT_ATTEMPTS = 10;
}
